#include <LEDA/graph.h>
#include <LEDA/graph_alg.h>
#include <LEDA/sweep_segments.h>

#include <LEDA/window.h>


void main()
{
  window W;
  panel P;

  GRAPH(point,int) G;
  node v,w;
  edge e;

  list(segment) L;
  list(segment) L0;



  int N = 40;
  int node_width = 3;
  int edge_width = 1;

  P.int_item("# segments",N);
  P.int_item("node width",node_width,1,8);
  P.int_item("edge width",edge_width,1,8);

  for(;;)
  {
    P.open();

    W.set_node_width(node_width);
    W.set_line_width(edge_width);
    W.init(-1000,1000,-1000);
  
  
    while (N--) 
    { double x1 = random(-990,0);
      double y1 = random(-990,900);
      double x2 = random(0,990);
      double y2 = random(-990,900);
      segment s(x1,y1,x2,y2);
      W << s;
      L.append(s);
     }
  
    W.del_message();
    W.message("SWEEPING ...");
    
    SWEEP_SEGMENTS(L,L0,G);
  
    W.clear();
    forall_nodes(v,G) W.draw_node(G[v]);
    forall_edges(e,G)  
     { v = source(e);
       w = target(e);
       W.draw_edge(G[v],G[w],blue);
      }
  
    W.del_message();
    W.message(string("PLANARITY TEST  N = %d",G.number_of_nodes()));
  
    if ( ! PLANAR(G) )
    { W.acknowledge("Graph is not planar");
      continue;
     }
  
  
     node_array(int) x(G),y(G);
  
  
     W.del_message();
     W.message("STRAIGHT LINE EMBEDDING");
  
     STRAIGHT_LINE_EMBEDDING(G,x,y);
     
     int xmax=0;
  
     forall_nodes(w,G)  
       if (x[w]>xmax) xmax= x[w];
     
     
     W.init(-5,xmax+5,-5);
  
     forall_nodes(v,G) W.draw_node(x[v],2*y[v]);
     forall_edges(e,G)  
     { v = source(e);
       w = target(e);
       W.draw_edge(x[v],2*y[v],x[w],2*y[w],red);
      }
     
      W.read_mouse();

    }
     
 }
